/*
  PS2Keyboard.h - PS2Keyboard library
  Copyright (c) 2007 Free Software Foundation.  All right reserved.
  Written by Christian Weichel <info@32leaves.net>

  ** Mostly rewritten Paul Stoffregen <paul@pjrc.com>, June 2010
  ** Modified for use with Arduino 13 by L. Abraham Smith, <n3bah@microcompdesign.com> * 
  ** Modified for easy interrup pin assignement on method begin(datapin,irq_pin). Cuningan <cuninganreset@gmail.com> **

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Edit par Krapoutchik le 9/06/2014.
  Le mappage des touches ne fonctionne pas pour des valeurs supérieures à 127. De 128 à 255
  la touche renvoie 2 caractères négatifs   FFFFFFC3 (-61) et FFFFFFBF (-65) lorsque qu'on mappe
  avec 255. Testé avec  la version 1.0.5 -R2 sur un Arduino Duemilanove.
  Pour l'APPLE 2 europlus,j'ai désactivé toutes les touches spéciales.
  
*/


#ifndef PS2Keyboard_h
#define PS2Keyboard_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h" // for attachInterrupt, FALLING
#else
#include "WProgram.h"
#endif

#include "utility/int_pins.h"

// Every call to read() returns a single byte for each
// keystroke.  These configure what byte will be returned
// for each "special" key.  To ignore a key, use zero.
#define PS2_TAB				9
#define PS2_ENTER			13
#define PS2_BACKSPACE			127
#define PS2_ESC				27
#define PS2_INSERT			0
#define PS2_DELETE			127
#define PS2_HOME			0
#define PS2_END				0
#define PS2_PAGEUP			25
#define PS2_PAGEDOWN			26
#define PS2_UPARROW			11
#define PS2_LEFTARROW			8
#define PS2_DOWNARROW			10
#define PS2_RIGHTARROW			21
#define PS2_F1				0
#define PS2_F2				0
#define PS2_F3				0
#define PS2_F4				0
#define PS2_F5				0
#define PS2_F6				0
#define PS2_F7				0
#define PS2_F8				0
#define PS2_F9				0
#define PS2_F10				0
#define PS2_F11				0
#define PS2_F12				0
#define PS2_SCROLL			255 // on devrait renvoyer 255 mais on renvoie  -61 et -65 bug ?

#define PS2_INVERTED_EXCLAMATION	0 //161 ¡
#define PS2_CENT_SIGN			0 // 162 ¢
#define PS2_POUND_SIGN			0 // 163£
#define PS2_CURRENCY_SIGN		0 //164 ¤
#define PS2_YEN_SIGN			0 // 165 ¥
#define PS2_BROKEN_BAR                 	0 //166 ¦
#define PS2_SECTION_SIGN		0 // 167§
#define PS2_DIAERESIS			0 //168 ¨
#define PS2_COPYRIGHT_SIGN		0 //169 ©
#define PS2_FEMININE_ORDINAL		0 //170 ª
#define PS2_LEFT_DOUBLE_ANGLE_QUOTE	0 //171 «
#define PS2_NOT_SIGN			0 //172 ¬
#define PS2_HYPHEN			0 // 173
#define PS2_REGISTERED_SIGN		0 // 174®
#define PS2_MACRON			0 //175 ¯
#define PS2_DEGREE_SIGN			0 // 176°
#define PS2_PLUS_MINUS_SIGN		0 //177 ±
#define PS2_SUPERSCRIPT_TWO		0 //178 ²
#define PS2_SUPERSCRIPT_THREE		0 //179 ³
#define PS2_ACUTE_ACCENT		0 //180 ´
#define PS2_MICRO_SIGN			0 //181 µ
#define PS2_PILCROW_SIGN		0 //182 ¶
#define PS2_MIDDLE_DOT			0 //183 ·
#define PS2_CEDILLA			0 //184 ¸
#define PS2_SUPERSCRIPT_ONE		0 //185 ¹
#define PS2_MASCULINE_ORDINAL		0 //186 º
#define PS2_RIGHT_DOUBLE_ANGLE_QUOTE	0 //187 »
#define PS2_FRACTION_ONE_QUARTER	0 //188 ¼
#define PS2_FRACTION_ONE_HALF		0 //189 ½
#define PS2_FRACTION_THREE_QUARTERS	0 //190 ¾
#define PS2_INVERTED_QUESTION MARK	0 //191 ¿
#define PS2_A_GRAVE			0 // 192À
#define PS2_A_ACUTE			0 //193 Á
#define PS2_A_CIRCUMFLEX		0 //194 Â
#define PS2_A_TILDE			0 // 195Ã
#define PS2_A_DIAERESIS			0 //196 Ä
#define PS2_A_RING_ABOVE		0 //197 Å
#define PS2_AE				0 //198 Æ
#define PS2_C_CEDILLA			0 //199 Ç
#define PS2_E_GRAVE			0 //200 È
#define PS2_E_ACUTE			0 //201 É
#define PS2_E_CIRCUMFLEX		0 //202 Ê
#define PS2_E_DIAERESIS			0 //203 Ë
#define PS2_I_GRAVE			0 //204 Ì
#define PS2_I_ACUTE			0 // 205Í
#define PS2_I_CIRCUMFLEX		0 //206 Î
#define PS2_I_DIAERESIS			0 //207 Ï
#define PS2_ETH				0 //208 Ð
#define PS2_N_TILDE			0 //209 Ñ
#define PS2_O_GRAVE			0 //210 Ò
#define PS2_O_ACUTE			0 // 211Ó
#define PS2_O_CIRCUMFLEX		0 //212 Ô
#define PS2_O_TILDE			0 // 213Õ
#define PS2_O_DIAERESIS			0 //214 Ö
#define PS2_MULTIPLICATION		0 //215 ×
#define PS2_O_STROKE			0 //216 Ø
#define PS2_U_GRAVE			0 //217 Ù
#define PS2_U_ACUTE			0 // 218Ú
#define PS2_U_CIRCUMFLEX		0 //219 Û
#define PS2_U_DIAERESIS			0 // 220Ü
#define PS2_Y_ACUTE			0 // 221Ý
#define PS2_THORN			0 //222 Þ
#define PS2_SHARP_S			0 //223 ß
#define PS2_a_GRAVE			0 // 224à
#define PS2_a_ACUTE			0 // 225á
#define PS2_a_CIRCUMFLEX		0 // 226â
#define PS2_a_TILDE			0 // 227ã
#define PS2_a_DIAERESIS			0 //228 ä
#define PS2_a_RING_ABOVE		0 //229 å
#define PS2_ae				0 //230 æ
#define PS2_c_CEDILLA			0 // 231ç
#define PS2_e_GRAVE			0 //232 è
#define PS2_e_ACUTE			0 //233 é
#define PS2_e_CIRCUMFLEX		0 //234 ê
#define PS2_e_DIAERESIS			0 //235 ë
#define PS2_i_GRAVE			0 //236 ì
#define PS2_i_ACUTE			0 //237 í
#define PS2_i_CIRCUMFLEX		0 //238 î
#define PS2_i_DIAERESIS			0 //239 ï
#define PS2_eth				0 // 240ð
#define PS2_n_TILDE			0 //241 ñ
#define PS2_o_GRAVE			0 //242 ò
#define PS2_o_ACUTE			0 // 243ó
#define PS2_o_CIRCUMFLEX		0 //244 ô
#define PS2_o_TILDE			0 //245 õ
#define PS2_o_DIAERESIS			0 //246 ö
#define PS2_DIVISION			0 // 247÷
#define PS2_o_STROKE			0 //248 ø
#define PS2_u_GRAVE			0 //249 ù
#define PS2_u_ACUTE			0 //250 ú
#define PS2_u_CIRCUMFLEX		0 // 251û
#define PS2_u_DIAERESIS			0 // 252ü
#define PS2_y_ACUTE			0 //256 ý
#define PS2_thorn			0 // 254þ
#define PS2_y_DIAERESIS			255 // ÿ


#define PS2_KEYMAP_SIZE 136

typedef struct {
	uint8_t noshift[PS2_KEYMAP_SIZE];
	uint8_t shift[PS2_KEYMAP_SIZE];
	uint8_t uses_altgr;
	uint8_t altgr[PS2_KEYMAP_SIZE];
} PS2Keymap_t;


extern const PROGMEM PS2Keymap_t PS2Keymap_US;
extern const PROGMEM PS2Keymap_t PS2Keymap_German;
extern const PROGMEM PS2Keymap_t PS2Keymap_French;


/**
 * Purpose: Provides an easy access to PS2 keyboards
 * Author:  Christian Weichel
 */
class PS2Keyboard {
  public:
  	/**
  	 * This constructor does basically nothing. Please call the begin(int,int)
  	 * method before using any other method of this class.
  	 */
    PS2Keyboard();
    
    /**
     * Starts the keyboard "service" by registering the external interrupt.
     * setting the pin modes correctly and driving those needed to high.
     * The propably best place to call this method is in the setup routine.
     */
    static void begin(uint8_t dataPin, uint8_t irq_pin, const PS2Keymap_t &map = PS2Keymap_US);
    
    /**
     * Returns true if there is a char to be read, false if not.
     */
    static bool available();
    
    /**
     * Returns the char last read from the keyboard.
     * If there is no char availble, -1 is returned.
     */
    static int read();
};

#endif
